/*
 * RT6_radarconvert.CMD  by mira308sw
 *
 * read radar coordinate from file "/BD0/SOURCE.ASC"
 * convert to RadarViewer format and write to file "/BD0/RADARS.CSV"
 *
 * source file must be in the following format:
 *   LON,LAT,TXT_1,TXT_2,SPEED 
 *
 * v:01.00  23/10/2013
 */

/* Drive letters
 */
#define DRIVE_USB       "/bd0"
#define DRIVE_HDD       "/SDC"
#define DRIVE_CD        "/D"
#define DRIVE_TFFS      "/F"
#define DRIVE_NAV       "/SDH"
#define DRIVE_USER      "/SDI"

#define FILE_SOURCE             DRIVE_USB "/source.asc"
#define FILE_RADARS             DRIVE_USB "/radars.csv"

/* Functions and data types from Integrated C Interpreter
 */

typedef unsigned char           BYTE;
typedef void                    FILE;

#define TRUE                    1
#define FALSE                   0
#define NO                      0
#define YES                     1
#define NULL                    (void*)0

typedef enum { ERROR=-1,OK=0 }  STATUS;

FILE*   fopen                   ( char *fn,char *mode );
STATUS  fclose                  ( FILE *f );
STATUS  fputs                   ( char *s,FILE *f );

STATUS  GetFileSize                   ( char *fn, int *size);
void*   UPGFindSymbol                 ( char *sym );
STATUS  ClearScreen                   ( void );
STATUS  ShowNavigationQuestionScreen  ( char *txt, int *answ );
int     AddUPGHistoryCommentary       ( void *funptr,... );   // max 7 parameters
STATUS  ShowNavigationBargraph        ( void );
STATUS  StepBargraph                  ( char *txt, int per );


/* extra C standard function, pull out through modified Builtins.out
 */
void *pf_atoi;
void *pf_sprintf;
void *pf_fprintf;
void *pf_ftell;
void *pf_sscanf;
void *pf_fgets;
void *pf_strchr;
void *pf_ConvertWGStoRelativeUTM;
void *pf_ConvertRelativeUTMtoWGS;


/* these functions work only on modified Builtins.out crc=19bb6041
 */
#define GFC                             AddUPGHistoryCommentary
#define atoi( s )                       GFC( pf_atoi,s )
#define strchr( str,chr )        (char*)GFC( pf_strchr,str,chr )
#define fgets( s,n,f )           (char*)GFC( pf_fgets,s,n,f )
#define ftell( f )                      GFC( pf_ftell,f )
#define CONVERTWGSTORELATIVEUTM( w,u )  GFC( pf_ConvertWGStoRelativeUTM,w,u )
#define CONVERTRELATIVEUTMTOWGS( u,w )  GFC( pf_ConvertRelativeUTMtoWGS,u,w )


STATUS InitFunctionPtr ( void )
{
  int   l_iAnsw;

  pf_atoi     =UPGFindSymbol( "atoi" );
  pf_sprintf  =UPGFindSymbol( "sprintf" );
  pf_fprintf  =UPGFindSymbol( "fprintf" );
  pf_sscanf   =UPGFindSymbol( "sscanf" );
  pf_fgets    =UPGFindSymbol( "fgets" );
  pf_ftell    =UPGFindSymbol( "ftell" );
  pf_strchr   =UPGFindSymbol( "strchr" );
  pf_ConvertWGStoRelativeUTM=UPGFindSymbol( "ConvertWGSToRelativeUTM__13C_NAV_UTILITYRC14TYPE_WGS_COORDR14TYPE_GEO_COORD" );
  pf_ConvertRelativeUTMtoWGS=UPGFindSymbol( "ConvertRelativeUTMToWGS__13C_NAV_UTILITYRC14TYPE_GEO_COORDR14TYPE_WGS_COORD" );

  if( pf_atoi                   !=NULL &&
      pf_sprintf                !=NULL &&
      pf_fprintf                !=NULL &&
      pf_sscanf                 !=NULL &&
      pf_ftell                  !=NULL &&
      pf_fgets                  !=NULL &&
      pf_strchr                 !=NULL &&
      pf_ConvertWGStoRelativeUTM!=NULL &&
      pf_ConvertRelativeUTMtoWGS!=NULL )
  {
      return OK;
  }

  ShowNavigationQuestionScreen( "InitFunctionPtr() FAILED!", &l_iAnsw );
  return ERROR;
}

typedef struct
{
  float lat,lon;
} t_wgs;

typedef struct
{
  int   X,Y;
} t_utm;

double PI;

void  setPI ( void )
{
  BYTE  *p=(BYTE*)&PI;
  p[0]=0x40;
  p[1]=0x09;
  p[2]=0x21;
  p[3]=0xFB;
  p[4]=0x54;
  p[5]=0x44;
  p[6]=0x2D;
  p[7]=0x18;
}

int  main ( int argc, char **argv )
{
  FILE *fin;
  FILE *fout;
  t_wgs wgs;
  t_utm utm;
  double  lat,lon;
  int  answ = FALSE;
  char line[256];
  int   SS,MS,LS,X,Y,x,y;
  char  *comma1,*comma2,*comma3,*comma4;
  int   speed;
  int   fsize;
  int   perc=0,p;

  setPI();
  utm.X = 0;
  utm.Y = 0;

  if( InitFunctionPtr()==OK )
  {
    GetFileSize( FILE_SOURCE,&fsize );

    fin =fopen( FILE_SOURCE,"rb" );
    fout=fopen( FILE_RADARS,"wb" );

    if( fin==NULL || fout==NULL )
      ShowNavigationQuestionScreen( "Errors in fopen",&answ );
    else
    {
      ShowNavigationBargraph();
      StepBargraph( "in progress...",perc );
      fputs( "CoordX,CoordY,SS,MS,LS,X,Y,subcat,mph,speed,dir,Speed,Road,City\n",fout );

      while( fgets( line,sizeof( line )-1,fin )!=NULL )
      {
        if( GFC( pf_sscanf,line,"%lf,%lf,",&lon,&lat )!=2 )
          ShowNavigationQuestionScreen( line,&answ );
        else
        {
          p = 100*ftell( fin )/fsize;
          if( p>=perc+10 )
            StepBargraph( "in progress...",perc=p );

          comma1 = strchr( line    ,',' );
          comma2 = strchr( comma1+1,',' );
          comma3 = strchr( comma2+1,',' );
          comma4 = strchr( comma3+1,',' );
          *comma2 = 0;
          *comma3 = 0;
          *comma4 = 0;
          speed = atoi( comma4+1 );

          wgs.lon=lon*PI/180.0;
          wgs.lat=lat*PI/180.0;
          CONVERTWGSTORELATIVEUTM( &wgs,&utm );

          GFC( pf_fprintf,fout,"%d,%d,",utm.X,utm.Y );

          x = utm.X;
          y = utm.Y;
          X  = x % 16000;
          Y  = y % 16000;
          x /= 1000;
          y /= 1000;
          SS = ( x % 16 ) + 16 * ( y % 16 );
          x /= 16;
          y /= 16;
          MS = ( x % 16 ) + 16 * ( y % 16 );
          x /= 16;
          y /= 16;
          LS = x + 28 * y;  // Europe
//          LS = x + 22 * y;  // China
//          LS = x + 28 * y;  // South America

          GFC( pf_fprintf,fout,"%d,%d,%d,",SS,MS,LS );
          GFC( pf_fprintf,fout,"%d,%d,",X,Y );

          fputs( "0,",fout );                     // subcat,
          fputs( "0,",fout );                     // mph,
//          fputs( "1,",fout );                     // mph,
          GFC( pf_fprintf,fout,"%d,",speed );     // speed
          fputs( "360,",fout );                   // dir
          GFC( pf_fprintf,fout,"%d km/h,",speed );// Speed (line 1)
//          GFC( pf_fprintf,fout,"%d mph,",speed ); // Speed (line 1)
          fputs( comma3+1,fout );                 // Road (line 2)
          fputs( ",",fout );
          fputs( comma2+1,fout );                 // City (line 3)
          fputs( "\n",fout );
        }
      }
      ShowNavigationQuestionScreen( "done",&answ );
    }
    fclose( fout );
    fclose( fin );
  }
  ClearScreen();
  return OK;
}

